package net.bigeon.gclc.process.io;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.junit.Test;

import net.bigeon.gclc.manager.PipedConsoleOutput;

public class ConnectingConsoleOutputTest {

    @Test
    public void testConnect() throws IOException {
        final ConnectingConsoleOutput out = new ConnectingConsoleOutput(
                ConnectingConsoleOutput.QUEUE, 10);
        final PipedConsoleOutput output = new PipedConsoleOutput();
        out.println("Some message");
        assertFalse("Message should not be transmitted", output.available());
        // Connect
        out.connect(output);

        // Connected outputs should receive data
        final String res = output.readNextLine();
        assertEquals("Message should be correctly transmitted", "Some message", res);
    }

    @Test
    public void testDisconnect() throws IOException {
        final ConnectingConsoleOutput out = new ConnectingConsoleOutput(
                ConnectingConsoleOutput.QUEUE, 10);
        final PipedConsoleOutput output = new PipedConsoleOutput();
        // NOP disconnection when no connection
        out.disconnect();
        // Connect
        out.connect(output);

        // Connected outputs should receive data
        out.println("Some message");
        final String res = output.readNextLine();
        assertEquals("Message should be correctly transmitted", "Some message", res);
        out.disconnect();
        out.println("Some message");
        assertFalse("Message should not be transmitted", output.available());
    }

    @Test
    public void testIsClosed() throws IOException {
        final ConnectingConsoleOutput out = new ConnectingConsoleOutput(
                ConnectingConsoleOutput.QUEUE, 10);
        final PipedConsoleOutput output = new PipedConsoleOutput();
        assertFalse("New output should not be closed", out.isClosed());
        // Connect
        out.connect(output);

        output.close();

        assertFalse("Output should not close when connected outputs do", out.isClosed());
        final PipedConsoleOutput output2 = new PipedConsoleOutput();
        out.connect(output2);
        out.close();
        assertTrue("Output should close on request", out.isClosed());
        assertFalse("Output should not close connected output when they do",
                output2.isClosed());
    }

    @Test
    public void testPrint() throws IOException {
        final ConnectingConsoleOutput out = new ConnectingConsoleOutput(
                ConnectingConsoleOutput.PERSIST, 10);

        out.print("Some text");
        out.println();
        final PipedConsoleOutput output = new PipedConsoleOutput();
        out.connect(output);
        String res = output.readNextLine();
        assertEquals("Message should be correctly transmitted", "Some text", res);
        out.disconnect();

        // Persist should be reprinted on any connect
        out.connect(output);
        res = output.readNextLine();
        assertEquals("Message should be correctly transmitted", "Some text", res);
        out.disconnect();
    }
}
